# packages load
import dash
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input,Output
from plotly.subplots import make_subplots
import plotly.graph_objects as go
import pandas as pd

# data load
cdata = pd.read_csv('data/cars.csv')

# inicializacia Dash aplikacie
app = dash.Dash(__name__)
app.config.suppress_callback_exceptions = True

# zoznam slovnikov label : value = oznacenie auta : oznacenie auta
def selector_options(carsdata):
    dlist = []
    for i in carsdata:
        dlist.append({'label' : i,'value' : i})
    return dlist

# zoznam slovnikov label : value = vlastnost auta : vlastnost auta
def features_options():
    dlist = []
    a = 0
    for i in cdata.columns:
        if a > 2: # 1. stlpec = vyrobca auta, 2. stlpec = oznacenie auta, 3. a dalsie stlpce = vlastnosti auta
            dlist.append({'label' : i,'value' : i})
        a += 1
    return dlist

# definicia Dash layoutu
app.layout = html.Div(
    # 1 riadok
    children = [html.Div(className = 'row',
                         children = [# 1. stlpec
                                     html.Div(className = 'four columns div-user-controls',
                                              children = [html.H3('Detailné porovnanie vlastností áut'),
                                                          html.P('Vyber jedno alebo viacero áut zo zoznamu:'),
                                                          html.Div(className = 'div-for-dropdown',
                                                                   children = [dcc.Dropdown(className = 'carssselector',
                                                                                            id = 'carsselector',
                                                                                            options = selector_options(cdata['označenie']),
                                                                                            multi = True,
                                                                                            value = '')
                                                                   ],
                                                          ),
                                                          html.P(''),
                                                          html.P('Vyber jednu alebo viacero vlastností áut zo zoznamu:'),
                                                          html.Div(className = 'div-for-dropdown',
                                                                   children = [dcc.Dropdown(className = 'featuresselector',
                                                                                            id = 'featuresselector',
                                                                                            options = features_options(),
                                                                                            multi = True,
                                                                                            value = '')
                                                                   ],
                                                          )
                                              ]
                                     ),
                                     # 2. stlpec
                                     html.Div(className = 'eight columns div-for-charts bg-grey',
                                              children = [# 1 riadok
                                                          html.Div(className = 'row',
                                                                   children = [dcc.Graph(id = 'graphs')]
                                                          )
                                              ]
                                     )
                         ]
                )
    ]
)

# Callback funkcia pre Dropdown carsselector a featuresselector
@app.callback(Output('graphs','figure'),[Input('carsselector','value'),Input('featuresselector','value')])
def update_graph_object(selected_dropdown_value1,selected_dropdown_value2):
    # naplnenie zoznamu oznaceni vsetkych aut
    cars_list = []
    for i in cdata['označenie']:
        cars_list.append(i)
    # zoznam oznaceni a prisluchajucich indexov vybranych aut
    sel_cars_descr = []
    sel_cars_index = []
    # naplnenie zoznamov oznaceni a prisluchajucich indexov vybranych aut
    # pre vsetky vybrane oznacenia aut
    for cdescr in selected_dropdown_value1:
        a = 1
        for i in cars_list:
            # ak bolo konkretne oznacenie auta vybrane
            if i == cdescr:
                sel_cars_descr.append(i) # zoznam oznaceni vybranych aut
                sel_cars_index.append(a) # zoznam prisluchajucich indexov vybranych aut
            a += 1
    # ak nebolo vybrane ziadne oznacenie auta
    if len(sel_cars_descr) == 0:
        # prazdne graficke zobrazenie
        return { "layout" : go.Layout(xaxis = { 'visible' : False },yaxis = { 'visible' : False },
                                      template = 'plotly_dark',
                                      paper_bgcolor = 'rgba(0,0,0,0)',
                                      plot_bgcolor = 'rgba(0,0,0,0)')
        }
    # pocet, zoznam nazvov a hodnot vybranych vlastnosti vybranych aut
    fcount = 0
    fnames = []
    fvalues = []
    # zoznam grafickych zobrazeni (subplots)
    traces = []
    # pre vsetky vybrane vlastnosti aut
    for fname in selected_dropdown_value2:
        # zvysenie poctu vybranych vlastnosti
        fcount += 1
        # pridanie nazvu konkretnej vlastnosti do zoznamu
        fnames.append(fname)

        # naplnenie zoznamu hodnot konkretnej vlastnosti vybranych aut
        fvalues.clear()
        # pre vsetky indexy vybranych aut
        for i in sel_cars_index:
            fvalues.append(cdata[fname][i-1])

        # naplnenie zoznamu grafickych zobrazeni
        # x = zoznam oznaceni vybranych aut
        # y = zoznam hodnot konkretnej vlastnosti vybranych aut
        traces.append(go.Bar(x = sel_cars_descr,y = fvalues,
                             marker_color = ['#636EFA','#EF553B','#00CC96','#AB63FA','#FFA15A','#19D3F3','#FF6692','#B6E880','#FF97FF','#FECB52']))

    # ak nebola vybrana ziadna vlastnost vybranych aut
    if fcount == 0:
        # prazdne graficke zobrazenie
        return { "layout" : go.Layout(xaxis = { 'visible' : False },yaxis = { 'visible' : False },
                                      template = 'plotly_dark',
                                      paper_bgcolor = 'rgba(0,0,0,0)',
                                      plot_bgcolor = 'rgba(0,0,0,0)')
        }

    # vytvorenie grafickych zobrazeni podla poctu vybranych vlastnosti
    # nazov grafickeho zobrazenia = nazov konkretnej vlastnosti
    fig = make_subplots(rows = fcount,cols = 1,subplot_titles = fnames)

    # naplnenie zoznamu grafickych zobrazeni (subplots)
    i = 0
    while i < fcount:
        fig.add_trace(traces[i],row = i+1,col = 1)
        i += 1
    # nastavenie stylu grafickych zobrazeni
    fig.update_layout(template = 'plotly_dark',
                      height = 400 * fcount,
                      paper_bgcolor = 'rgba(0,0,0,0)',
                      plot_bgcolor = 'rgba(0,0,0,0)',
                      title = {'text' : '','font' : {'color' : 'white'},'x' : 0.5},
                      showlegend = False)

    # aktualizacia dcc.Graph(figure)
    return fig

# spustenie Dash aplikacie
if __name__ == '__main__':
    app.run_server(debug = True)